{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Módulos\n",
    "\n",
    "En JavaScript, los módulos son colecciones de datos y funciones útiles para el programa principal. Los módulos permiten encapsular el código, lo que ayuda a mantenerlo organizado y fácil de mantener. Además, siguen el Principio de Menor Exposición (POLE), lo que significa que solo exponen una API pública y pueden contener detalles privados. Los módulos pueden ser stateful, es decir, pueden guardar información a lo largo del tiempo.\n",
    "\n",
    "### Problemas sin módulos\n",
    "\n",
    "Cuando el código está en varios archivos .js sin usar módulos, el espacio de nombres global se llena de variables y funciones. Esto puede causar conflictos y errores difíciles de depurar. Además, el orden de los archivos importa, ya que se interpretan como si todo el código estuviera en un único archivo.\n",
    "\n",
    "### Espacios de nombres y estructuras de datos que no son módulos\n",
    "\n",
    "**Ejemplo de espacio de nombres (no es un módulo):**\n",
    "```javascript\n",
    "var Utils = {\n",
    "   cancelEvt(evt) {\n",
    "       evt.preventDefault(); \n",
    "       evt.stopPropagation(); \n",
    "       evt.stopImmediatePropagation();\n",
    "   },\n",
    "   wait(ms) {\n",
    "       return new Promise(function c(res){ setTimeout(res,ms); });\n",
    "   },\n",
    "   isValidEmail(email) {\n",
    "       return /[^@]+@[^@.]+\\.[^@.]+/.test(email);\n",
    "   }\n",
    "};\n",
    "```\n",
    "En este ejemplo, `Utils` es un objeto que agrupa funciones relacionadas. Sin embargo, no es un módulo porque no encapsula el código ni oculta detalles internos.\n",
    "\n",
    "**Ejemplo de estructura de datos (no es un módulo):**\n",
    "```javascript\n",
    "var Student = {\n",
    "   records: [\n",
    "      { id: 14, name: \"Kyle\", grade: 86 }, \n",
    "      { id: 73, name: \"Suzy\", grade: 87 }, \n",
    "      { id: 112, name: \"Frank\", grade: 75 }, \n",
    "      { id: 6, name: \"Sarah\", grade: 91 }\n",
    "   ],\n",
    "   getName(studentID) {\n",
    "       var student = this.records.find(student => student.id == studentID);\n",
    "       return student.name;\n",
    "   }\n",
    "};\n",
    "Student.getName(73); // Suzy\n",
    "```\n",
    "Aquí, `Student` es una estructura de datos que contiene un array de registros y una función para obtener el nombre de un estudiante. Similar al ejemplo anterior, no encapsula ni protege los datos.\n",
    "\n",
    "### Módulos antes de ES6\n",
    "\n",
    "Antes de ES6, se usaban diferentes estándares para crear módulos en JavaScript, como CommonJS, Asynchronous Module Definition (AMD) y Universal Module Definition (UMD). Sin embargo, ninguno de estos era nativo de JavaScript y requerían bibliotecas adicionales. También se utilizaron herramientas como Babel o Webpack para traducir el código en módulos a código sin módulos.\n",
    "\n",
    "### Módulos manuales (con estado y control de acceso)\n",
    "\n",
    "Antes de ES6, se podían crear módulos manualmente para encapsular código y controlar el acceso a los datos. \n",
    "\n",
    "**Ejemplo de módulo manual:**\n",
    "```javascript\n",
    "function defineStudent() {\n",
    "   var records = [\n",
    "      { id: 14, name: \"Kyle\", grade: 86 }, \n",
    "      { id: 73, name: \"Suzy\", grade: 87 }, \n",
    "      { id: 112, name: \"Frank\", grade: 75 }, \n",
    "      { id: 6, name: \"Sarah\", grade: 91 }\n",
    "   ];\n",
    "   var publicAPI = { getName };\n",
    "   return publicAPI;\n",
    "\n",
    "   function getName(studentID) {\n",
    "       var student = records.find(student => student.id == studentID);\n",
    "       return student.name;\n",
    "   }\n",
    "}\n",
    "var fullTime = defineStudent();\n",
    "fullTime.getName(73); // Suzy\n",
    "```\n",
    "En este ejemplo, `defineStudent` es una función que define un módulo. El array `records` y la función `getName` están encapsulados dentro del módulo, y solo `getName` se expone públicamente.\n"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "\n",
    "### Módulos ES6\n",
    "\n",
    "https://developer.mozilla.org/en-US/docs/Web/JavaScript/Guide/Modules \n",
    "\n",
    "Con la introducción de ES6, JavaScript nativo ahora admite módulos a través de las palabras clave `import` y `export`. Los módulos ES6 no agregan nada al ámbito global, siempre están en modo estricto y requieren un servidor para cargar los archivos a través de HTTP.\n",
    "\n",
    "**Definición y exportación de módulos ES6:**\n",
    "```javascript\n",
    "// students.js\n",
    "export function getName(studentID) {\n",
    "   var records = [\n",
    "      { id: 14, name: \"Kyle\", grade: 86 }, \n",
    "      { id: 73, name: \"Suzy\", grade: 87 }, \n",
    "      { id: 112, name: \"Frank\", grade: 75 }, \n",
    "      { id: 6, name: \"Sarah\", grade: 91 }\n",
    "   ];\n",
    "   var student = records.find(student => student.id == studentID);\n",
    "   return student.name;\n",
    "}\n",
    "```\n",
    "\n",
    "**Importación de módulos ES6:**\n",
    "```javascript\n",
    "// main.js\n",
    "import { getName } from \"/path/to/students.js\";\n",
    "console.log(getName(73)); // Suzy\n",
    "```\n",
    "\n",
    "**Exportar múltiples funciones:**\n",
    "```javascript\n",
    "// students.js\n",
    "export function getName(studentID) { /*...*/ }\n",
    "export function getGrade(studentID) { /*...*/ }\n",
    "```\n",
    "\n",
    "**Importar múltiples funciones:**\n",
    "```javascript\n",
    "// main.js\n",
    "import { getName, getGrade } from \"/path/to/students.js\";\n",
    "```\n",
    "\n",
    "**Exportación e importación por defecto:**\n",
    "```javascript\n",
    "// students.js\n",
    "export default function getName(studentID) { /*...*/ }\n",
    "\n",
    "// main.js\n",
    "import getName from \"/path/to/students.js\";\n",
    "```\n",
    "\n",
    "**Importación con alias:**\n",
    "```javascript\n",
    "// main.js\n",
    "import { getName as getStudentName } from \"/path/to/students.js\";\n",
    "```\n",
    "\n",
    "**Importación de todo el módulo:**\n",
    "```javascript\n",
    "// main.js\n",
    "import * as Student from \"/path/to/students.js\";\n",
    "Student.getName(73); // Suzy\n",
    "```\n",
    "\n",
    "### Uso de `<script>` con módulos\n",
    "\n",
    "Se pueden cargar módulos en un archivo HTML usando el atributo `type=\"module\"` en las etiquetas `<script>`.\n",
    "\n",
    "**Ejemplo:**\n",
    "```html\n",
    "<!DOCTYPE html>\n",
    "<html>\n",
    "<head>\n",
    "   <title>Módulos ES6</title>\n",
    "</head>\n",
    "<body>\n",
    "   <script type=\"module\" src=\"functions.js\"></script>\n",
    "   <script type=\"module\" src=\"script.js\"></script>\n",
    "</body>\n",
    "</html>\n",
    "```"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### Importación asíncrona\n",
    "\n",
    "https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Operators/import\n",
    "\n",
    "* Devuelve una promesa (es asíncrono)\n",
    "* Se puede importar Javascript en \"tiempo real\"\n",
    "* No está limitado a usar el import al inicio\n",
    "* Puede ejecutar código sólo en ciertos casos"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### Importar en `Bare Names`\n",
    "\n",
    "En `Vite` y otros entornos, no es necesario especificar la extensión de los archivos al importar, incluso si están en `node_modules`no hace falta ni la ruta. Esto es por dos razones:\n",
    "\n",
    "* Vite usa un sistema de resolución de módulos que resuelve automáticamente las extensiones de archivos. Esto se puede configurar con `resolve.extensions` \n",
    "\n",
    "* Las bibliotecas instaladas en `node_modules` tienen una referencia en `package.json` y su propio package.json indica cómo se exporta. Vite y otros como Webpack siguen el estándar ESM, pero se necesita un proceso como el que hacen al hacer el `build`que las resuelva. \n",
    "\n",
    "Si queremos imitar esto sin usa Vite o similar, necesitaríamos `import maps` https://developer.mozilla.org/en-US/docs/Web/JavaScript/Guide/Modules#importing_modules_as_bare_names \n",
    "\n"
   ]
  }
 ],
 "metadata": {
  "language_info": {
   "name": "python"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 2
}
